@@ -54,7 +54,8 @@ class RestAlarmNotifier(notifier.AlarmNotifier):
     """Rest alarm notifier."""
 
     @staticmethod
-    def notify(action, alarm_id, previous, current, reason, reason_data):
+    def notify(action, alarm_id, previous, current, reason, reason_data,
+               headers=None):
         LOG.info(_(
             "Notifying alarm %(alarm_id)s from %(previous)s "
             "to %(current)s with action %(action)s because "
@@ -64,7 +65,10 @@ class RestAlarmNotifier(notifier.AlarmNotifier):
         body = {'alarm_id': alarm_id, 'previous': previous,
                 'current': current, 'reason': reason,
                 'reason_data': reason_data}
-        kwargs = {'data': jsonutils.dumps(body)}
+        headers = headers or {}
+        headers['content-type'] = 'application/json'
+        kwargs = {'data': jsonutils.dumps(body),
+                  'headers': headers}
 
         if action.scheme == 'https':
             default_verify = int(cfg.CONF.alarm.rest_notifier_ssl_verify)
