@@ -18,14 +18,12 @@
 
 """Access Control Lists (ACL's) control access the API server."""
 
+from ceilometer.openstack.common import policy
 from keystoneclient.middleware import auth_token
 from oslo.config import cfg
-from pecan import hooks
-from webob import exc
-
-from ceilometer import policy
 
 
+_ENFORCER = None
 OPT_GROUP_NAME = 'keystone_authtoken'
 
 
@@ -46,11 +44,29 @@ def install(app, conf):
                                    conf=dict(conf.get(OPT_GROUP_NAME)))
 
 
-class AdminAuthHook(hooks.PecanHook):
-    """Verify that the user has admin rights
-    """
+def get_limited_to(headers):
+    """Return the user and project the request should be limited to.
 
-    def before(self, state):
-        headers = state.request.headers
-        if not policy.check_is_admin(headers.get('X-Roles', "").split(",")):
-            raise exc.HTTPUnauthorized()
+    :param headers: HTTP headers dictionary
+    :return: A tuple of (user, project), set to None if there's no limit on
+    one of these.
+
+    """
+    global _ENFORCER
+    if not _ENFORCER:
+        _ENFORCER = policy.Enforcer()
+    if not _ENFORCER.enforce('context_is_admin',
+                             {},
+                             {'roles': headers.get('X-Roles', "").split(",")}):
+        return headers.get('X-User-Id'), headers.get('X-Project-Id')
+    return None, None
+
+
+def get_limited_to_project(headers):
+    """Return the project the request should be limited to.
+
+    :param headers: HTTP headers dictionary
+    :return: A project, or None if there's no limit on it.
+
+    """
+    return get_limited_to(headers)[1]
