@@ -16,6 +16,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import logging
+import os
+import socket
+from wsgiref import simple_server
+
+import netaddr
 from oslo.config import cfg
 import pecan
 
@@ -23,13 +29,18 @@ from ceilometer.api import acl
 from ceilometer.api import config as api_config
 from ceilometer.api import hooks
 from ceilometer.api import middleware
-from ceilometer.api.v1 import app as v1app
+from ceilometer.openstack.common import log
+from ceilometer import storage
 
+LOG = log.getLogger(__name__)
 
 auth_opts = [
     cfg.StrOpt('auth_strategy',
                default='keystone',
                help='The strategy to use for auth: noauth or keystone.'),
+    cfg.BoolOpt('enable_v1_api',
+                default=True,
+                help='Deploy the deprecated v1 API.'),
 ]
 
 CONF = cfg.CONF
@@ -45,30 +56,31 @@ def get_pecan_config():
 def setup_app(pecan_config=None, extra_hooks=None):
     # FIXME: Replace DBHook with a hooks.TransactionHook
     app_hooks = [hooks.ConfigHook(),
-                 hooks.DBHook()]
+                 hooks.DBHook(
+                     storage.get_connection(cfg.CONF),
+                 ),
+                 hooks.PipelineHook(),
+                 hooks.TranslationHook()]
     if extra_hooks:
         app_hooks.extend(extra_hooks)
 
     if not pecan_config:
         pecan_config = get_pecan_config()
 
-    if pecan_config.app.enable_acl:
-        app_hooks.append(acl.AdminAuthHook())
-
     pecan.configuration.set_config(dict(pecan_config), overwrite=True)
 
     app = pecan.make_app(
         pecan_config.app.root,
         static_root=pecan_config.app.static_root,
         template_path=pecan_config.app.template_path,
-        logging=getattr(pecan_config, 'logging', {}),
-        debug=getattr(pecan_config.app, 'debug', False),
+        debug=CONF.debug,
         force_canonical=getattr(pecan_config.app, 'force_canonical', True),
         hooks=app_hooks,
         wrap_app=middleware.ParsableErrorMiddleware,
+        guess_content_type_from_ext=False
     )
 
-    if pecan_config.app.enable_acl:
+    if getattr(pecan_config.app, 'enable_acl', True):
         return acl.install(app, cfg.CONF)
 
     return app
@@ -79,10 +91,54 @@ class VersionSelectorApplication(object):
         pc = get_pecan_config()
         pc.app.debug = CONF.debug
         pc.app.enable_acl = (CONF.auth_strategy == 'keystone')
-        self.v1 = v1app.make_app(cfg.CONF, enable_acl=pc.app.enable_acl)
+        if cfg.CONF.enable_v1_api:
+            from ceilometer.api.v1 import app as v1app
+            self.v1 = v1app.make_app(cfg.CONF, enable_acl=pc.app.enable_acl)
+        else:
+            def not_found(environ, start_response):
+                start_response('404 Not Found', [])
+                return []
+            self.v1 = not_found
         self.v2 = setup_app(pecan_config=pc)
 
     def __call__(self, environ, start_response):
         if environ['PATH_INFO'].startswith('/v1/'):
             return self.v1(environ, start_response)
         return self.v2(environ, start_response)
+
+
+def get_server_cls(host):
+    """Return an appropriate WSGI server class base on provided host
+
+    :param host: The listen host for the ceilometer API server.
+    """
+    server_cls = simple_server.WSGIServer
+    if netaddr.valid_ipv6(host):
+        # NOTE(dzyu) make sure use IPv6 sockets if host is in IPv6 pattern
+        if getattr(server_cls, 'address_family') == socket.AF_INET:
+            class server_cls(server_cls):
+                address_family = socket.AF_INET6
+    return server_cls
+
+
+def build_server():
+    # Build the WSGI app
+    root = VersionSelectorApplication()
+
+    # Create the WSGI server and start it
+    host, port = cfg.CONF.api.host, cfg.CONF.api.port
+    server_cls = get_server_cls(host)
+    srv = simple_server.make_server(host, port, root, server_cls)
+
+    LOG.info(_('Starting server in PID %s') % os.getpid())
+    LOG.info(_("Configuration:"))
+    cfg.CONF.log_opt_values(LOG, logging.INFO)
+
+    if host == '0.0.0.0':
+        LOG.info(_(
+            'serving on 0.0.0.0:%(sport)s, view at http://127.0.0.1:%(vport)s')
+            % ({'sport': port, 'vport': port}))
+    else:
+        LOG.info(_("serving on http://%(host)s:%(port)s") % (
+                 {'host': host, 'port': port}))
+    return srv
