@@ -18,11 +18,11 @@
 # under the License.
 
 import threading
+
 from oslo.config import cfg
 from pecan import hooks
 
 from ceilometer import pipeline
-from ceilometer import transformer
 
 
 class ConfigHook(hooks.PecanHook):
@@ -36,12 +36,10 @@ class ConfigHook(hooks.PecanHook):
 
 class DBHook(hooks.PecanHook):
 
-    def __init__(self, storage_engine, storage_connection):
-        self.storage_engine = storage_engine
+    def __init__(self, storage_connection):
         self.storage_connection = storage_connection
 
     def before(self, state):
-        state.request.storage_engine = self.storage_engine
         state.request.storage_conn = self.storage_connection
 
 
@@ -56,9 +54,7 @@ class PipelineHook(hooks.PecanHook):
         if self.__class__.pipeline_manager is None:
             # this is done here as the cfg options are not available
             # when the file is imported.
-            self.__class__.pipeline_manager = pipeline.setup_pipeline(
-                transformer.TransformerExtensionManager(
-                    'ceilometer.transformer'))
+            self.__class__.pipeline_manager = pipeline.setup_pipeline()
 
     def before(self, state):
         state.request.pipeline_manager = self.pipeline_manager
