@@ -19,21 +19,35 @@
 """
 
 import abc
+import six
 
+from ceilometer.openstack.common import timeutils
 from ceilometer import plugin
 
 
+@six.add_metaclass(abc.ABCMeta)
 class ComputePollster(plugin.PollsterBase):
     """Base class for plugins that support the polling API on the compute node.
     """
 
-    __metaclass__ = abc.ABCMeta
-
     @abc.abstractmethod
-    def get_samples(self, manager, cache, instance):
+    def get_samples(self, manager, cache, resources):
         """Return a sequence of Counter instances from polling the resources.
 
         :param manager: The service manager invoking the plugin
         :param cache: A dictionary for passing data between plugins
-        :param instance: The instance to examine
+        :param resources: The resources to examine (expected to be instances)
         """
+
+    def _record_poll_time(self):
+        """Method records current time as the poll time.
+
+        :return: time in seconds since the last poll time was recorded
+        """
+        current_time = timeutils.utcnow()
+        duration = None
+        if hasattr(self, '_last_poll_time'):
+            duration = timeutils.delta_seconds(self._last_poll_time,
+                                               current_time)
+        self._last_poll_time = current_time
+        return duration
