@@ -13,13 +13,16 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import warnings
-
-from oslo_log import log
-from urllib import parse as urlparse
+import six
+from six import moves
+from six.moves.urllib import parse as url_parse
 
 from ceilometer.network.statistics import driver
 from ceilometer.network.statistics.opendaylight import client
+from ceilometer.openstack.common.gettextutils import _  # noqa
+from ceilometer.openstack.common import log
+from ceilometer.openstack.common import timeutils
+from ceilometer import utils
 
 
 LOG = log.getLogger(__name__)
@@ -28,7 +31,7 @@ LOG = log.getLogger(__name__)
 def _get_properties(properties, prefix='properties'):
     resource_meta = {}
     if properties is not None:
-        for k, v in properties.items():
+        for k, v in six.iteritems(properties):
             value = v['value']
             key = prefix + '_' + k
             if 'name' in v:
@@ -40,7 +43,7 @@ def _get_properties(properties, prefix='properties'):
 def _get_int_sample(key, statistic, resource_id, resource_meta):
     if key not in statistic:
         return None
-    return int(statistic[key]), resource_id, resource_meta
+    return (int(statistic[key]), resource_id, resource_meta)
 
 
 class OpenDayLightDriver(driver.Driver):
@@ -48,43 +51,36 @@ class OpenDayLightDriver(driver.Driver):
 
     This driver uses resources in "pipeline.yaml".
     Resource requires below conditions:
-
     * resource is url
     * scheme is "opendaylight"
 
     This driver can be configured via query parameters.
     Supported parameters:
-
     * scheme:
-      The scheme of request url to OpenDaylight REST API endpoint.
-      (default http)
+        The scheme of request url to OpenDaylight REST API endpoint.
+        (default http)
     * auth:
-      Auth strategy of http.
-      This parameter can be set basic and digest.(default None)
+        Auth strategy of http.
+        This parameter can be set basic and digest.(default None)
     * user:
-      This is username that is used by auth.(default None)
+        This is username that is used by auth.(default None)
     * password:
-      This is password that is used by auth.(default None)
+        This is password that is used by auth.(default None)
     * container_name:
-      Name of container of OpenDaylight.(default "default")
-      This parameter allows multi values.
+        Name of container of OpenDaylight.(default "default")
+        This parameter allows multi vaues.
 
-    e.g.::
+    e.g.
+        opendaylight://127.0.0.1:8080/controller/nb/v2?container_name=default&
+            container_name=egg&auth=basic&user=admin&password=admin&scheme=http
 
-      opendaylight://127.0.0.1:8080/controller/nb/v2?container_name=default&
-      container_name=egg&auth=basic&user=admin&password=admin&scheme=http
-
-    In this case, the driver send request to below URLs:
-
-      http://127.0.0.1:8080/controller/nb/v2/statistics/default/flow
-      http://127.0.0.1:8080/controller/nb/v2/statistics/egg/flow
+        In this case, the driver send request to below URL:
+            http://127.0.0.1:8080/controller/nb/v2/statistics/default/flow
+            and
+            http://127.0.0.1:8080/controller/nb/v2/statistics/egg/flow
     """
-    def __init__(self, conf):
-        warnings.warn('Support for OpenDaylight is deprecated.',
-                      category=DeprecationWarning, stacklevel=2)
-        super(OpenDayLightDriver, self).__init__(conf)
-
-    def _prepare_cache(self, endpoint, params, cache):
+    @staticmethod
+    def _prepare_cache(endpoint, params, cache):
 
         if 'network.statistics.opendaylight' in cache:
             return cache['network.statistics.opendaylight']
@@ -100,7 +96,7 @@ class OpenDayLightDriver(driver.Driver):
             odl_params['user'] = params['user'][0]
         if 'password' in params:
             odl_params['password'] = params['password'][0]
-        cs = client.Client(self.conf, endpoint, odl_params)
+        cs = client.Client(endpoint, odl_params)
 
         for container_name in container_names:
             try:
@@ -122,7 +118,7 @@ class OpenDayLightDriver(driver.Driver):
                 container_data['topology'] = cs.topology.get_topology(
                     container_name)
 
-                # get switch information
+                # get switch informations
                 container_data['switch'] = cs.switch_manager.get_nodes(
                     container_name)
 
@@ -145,7 +141,7 @@ class OpenDayLightDriver(driver.Driver):
                 container_data['user_links'] = user_links
                 for user_link_row in user_links_raw['userLinks']:
                     user_link = {}
-                    for k, v in user_link_row.items():
+                    for k, v in six.iteritems(user_link_row):
                         if (k == "dstNodeConnector" or
                                 k == "srcNodeConnector"):
                             port_raw, node_raw = v.split('@')
@@ -158,14 +154,17 @@ class OpenDayLightDriver(driver.Driver):
                     user_links.append(user_link)
 
                 # get link status to hosts
-                container_data['active_hosts'] = (
-                    cs.host_tracker.get_active_hosts(container_name))
-                container_data['inactive_hosts'] = (
-                    cs.host_tracker.get_inactive_hosts(container_name))
+                container_data['active_hosts'] = cs.host_tracker.\
+                    get_active_hosts(container_name)
+                container_data['inactive_hosts'] = cs.host_tracker.\
+                    get_inactive_hosts(container_name)
+
+                container_data['timestamp'] = timeutils.isotime()
+
                 data[container_name] = container_data
             except Exception:
-                LOG.exception('Request failed to connect to OpenDaylight'
-                              ' with NorthBound REST API')
+                LOG.exception(_('Request failed to connect to OpenDaylight'
+                                ' with NorthBound REST API'))
 
         cache['network.statistics.opendaylight'] = data
 
@@ -185,18 +184,19 @@ class OpenDayLightDriver(driver.Driver):
             # OpenDaylight REST API has not api to getting meter.
             return None
 
-        parts = urlparse.ParseResult(params.get('scheme', ['http'])[0],
-                                     parse_url.netloc,
-                                     parse_url.path,
-                                     None,
-                                     None,
-                                     None)
-        endpoint = urlparse.urlunparse(parts)
+        parts = url_parse.ParseResult(params.get('scheme', ['http'])[0],
+                                      parse_url.netloc,
+                                      parse_url.path,
+                                      None,
+                                      None,
+                                      None)
+        endpoint = url_parse.urlunparse(parts)
 
         data = self._prepare_cache(endpoint, params, cache)
 
         samples = []
-        for name, value in data.items():
+        for name, value in six.iteritems(data):
+            timestamp = value['timestamp']
             for sample in iter(extractor, value):
                 if sample is not None:
                     # set controller name and container name
@@ -204,7 +204,7 @@ class OpenDayLightDriver(driver.Driver):
                     sample[2]['controller'] = 'OpenDaylight'
                     sample[2]['container'] = name
 
-                    samples.append(sample + (None, ))
+                    samples.append(sample + (timestamp, ))
 
         return samples
 
@@ -232,7 +232,7 @@ class OpenDayLightDriver(driver.Driver):
 
         resource_meta.update(_get_properties(statistic.get('properties')))
 
-        return 1, resource_id, resource_meta
+        return (1, resource_id, resource_meta)
 
     @staticmethod
     def _iter_port(extractor, data):
@@ -289,7 +289,7 @@ class OpenDayLightDriver(driver.Driver):
             break
 
         # link status to hosts
-        for hosts, status in zip(
+        for hosts, status in moves.zip(
                 [data['active_hosts'], data['inactive_hosts']],
                 ['active', 'inactive']):
             for host_config in hosts['hostConfig']:
@@ -305,7 +305,7 @@ class OpenDayLightDriver(driver.Driver):
 
                 break
 
-        return 1, resource_id, resource_meta
+        return (1, resource_id, resource_meta)
 
     @staticmethod
     def _switch_port_receive_packets(statistic, resource_id,
@@ -390,7 +390,7 @@ class OpenDayLightDriver(driver.Driver):
 
     @staticmethod
     def _switch_table(statistic, resource_id, resource_meta):
-        return 1, resource_id, resource_meta
+        return (1, resource_id, resource_meta)
 
     @staticmethod
     def _switch_table_active_entries(statistic, resource_id,
@@ -410,36 +410,14 @@ class OpenDayLightDriver(driver.Driver):
         return _get_int_sample('matchedCount', statistic, resource_id,
                                resource_meta)
 
-    def dict_to_keyval(self, value, key_base=None):
-        """Expand a given dict to its corresponding key-value pairs.
-
-        Generated keys are fully qualified, delimited using dot notation.
-        ie. key = 'key.child_key.grandchild_key[0]'
-        """
-        val_iter, key_func = None, None
-        if isinstance(value, dict):
-            val_iter = value.items()
-            key_func = lambda k: key_base + '.' + k if key_base else k  # noqa
-        elif isinstance(value, (tuple, list)):
-            val_iter = enumerate(value)
-            key_func = lambda k: key_base + '[%d]' % k  # noqa: E731
-
-        if val_iter:
-            for k, v in val_iter:
-                key_gen = key_func(k)
-                if isinstance(v, dict) or isinstance(v, (tuple, list)):
-                    for key_gen, v in self.dict_to_keyval(v, key_gen):
-                        yield key_gen, v
-                else:
-                    yield key_gen, v
-
-    def _iter_flow(self, extractor, data):
+    @staticmethod
+    def _iter_flow(extractor, data):
         for flow_statistic in data['flow']['flowStatistics']:
             for statistic in flow_statistic['flowStatistic']:
                 resource_meta = {'flow_id': statistic['flow']['id'],
                                  'table_id': statistic['tableId']}
-                for key, value in self.dict_to_keyval(statistic['flow'],
-                                                      'flow'):
+                for key, value in utils.dict_to_keyval(statistic['flow'],
+                                                       'flow'):
                     resource_meta[key.replace('.', '_')] = value
                 yield extractor(statistic,
                                 flow_statistic['node']['id'],
@@ -447,7 +425,7 @@ class OpenDayLightDriver(driver.Driver):
 
     @staticmethod
     def _switch_flow(statistic, resource_id, resource_meta):
-        return 1, resource_id, resource_meta
+        return (1, resource_id, resource_meta)
 
     @staticmethod
     def _switch_flow_duration_seconds(statistic, resource_id,
