@@ -41,11 +41,12 @@ def logged(func):
 
 
 class Client(object):
+    """A client which gets information via python-novaclient."""
 
     def __init__(self):
-        """Returns a nova Client object."""
+        """Initialize a nova client object."""
         conf = cfg.CONF.service_credentials
-        tenant = conf.os_tenant_id and conf.os_tenant_id or conf.os_tenant_name
+        tenant = conf.os_tenant_id or conf.os_tenant_name
         self.nova_client = nova_client.Client(
             username=conf.os_username,
             api_key=conf.os_password,
@@ -54,6 +55,7 @@ class Client(object):
             region_name=conf.os_region_name,
             endpoint_type=conf.os_endpoint_type,
             cacert=conf.os_cacert,
+            insecure=conf.insecure,
             no_cache=True)
 
     def _with_flavor_and_image(self, instances):
@@ -101,7 +103,7 @@ class Client(object):
         image_metadata = getattr(image, 'metadata', None)
 
         for attr in ['kernel_id', 'ramdisk_id']:
-            ameta = image_metadata.get(attr, None) if image_metadata else None
+            ameta = image_metadata.get(attr) if image_metadata else None
             setattr(instance, attr, ameta)
 
     @logged
