@@ -14,20 +14,21 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from sqlalchemy import *
-
-meta = MetaData()
+from sqlalchemy import Column
+from sqlalchemy import Integer
+from sqlalchemy import MetaData
+from sqlalchemy import Table
 
 
 def upgrade(migrate_engine):
-    meta.bind = migrate_engine
+    meta = MetaData(bind=migrate_engine)
     meter = Table('meter', meta, autoload=True)
     duration = Column('counter_duration', Integer)
     meter.drop_column(duration)
 
 
 def downgrade(migrate_engine):
-    meta.bind = migrate_engine
+    meta = MetaData(bind=migrate_engine)
     meter = Table('meter', meta, autoload=True)
     duration = Column('counter_duration', Integer)
     meter.create_column(duration)
