@@ -21,16 +21,13 @@ from sqlalchemy import MetaData
 from sqlalchemy import Table
 
 
-meta = MetaData()
-
-
 def upgrade(migrate_engine):
-    meta.bind = migrate_engine
+    meta = MetaData(bind=migrate_engine)
     meter = Table('meter', meta, autoload=True)
     meter.c.counter_volume.alter(type=Float(53))
 
 
 def downgrade(migrate_engine):
-    meta.bind = migrate_engine
+    meta = MetaData(bind=migrate_engine)
     meter = Table('meter', meta, autoload=True)
     meter.c.counter_volume.alter(type=Integer)
