@@ -1,3 +1,4 @@
+# -*- encoding: utf-8 -*-
 
 # Copyright 2013 OpenStack Foundation
 # All Rights Reserved.
@@ -16,6 +17,7 @@
 # under the License.
 
 from sqlalchemy import MetaData
+from sqlalchemy import String
 from sqlalchemy import Table
 from sqlalchemy import Text
 
@@ -24,3 +26,9 @@ def upgrade(migrate_engine):
     meta = MetaData(bind=migrate_engine)
     resource = Table('resource', meta, autoload=True)
     resource.c.resource_metadata.alter(type=Text)
+
+
+def downgrade(migrate_engine):
+    meta = MetaData(bind=migrate_engine)
+    resource = Table('resource', meta, autoload=True)
+    resource.c.resource_metadata.alter(type=String(5000))
