@@ -1,3 +1,4 @@
+# -*- encoding: utf-8 -*-
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -22,3 +23,11 @@ def upgrade(migrate_engine):
     metadata_float.c.value.alter(type=Float(53))
     trait = Table('trait', meta, autoload=True)
     trait.c.t_float.alter(type=Float(53))
+
+
+def downgrade(migrate_engine):
+    meta = MetaData(bind=migrate_engine)
+    metadata_float = Table('metadata_float', meta, autoload=True)
+    metadata_float.c.value.alter(type=Float())
+    trait = Table('trait', meta, autoload=True)
+    trait.c.t_string.alter(type=Float())
