@@ -1,3 +1,4 @@
+# -*- encoding: utf-8 -*-
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -10,9 +11,10 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-from oslo_utils import timeutils
+
 import sqlalchemy
 
+from ceilometer.openstack.common import timeutils
 from ceilometer.storage.sqlalchemy import models
 
 
@@ -22,3 +24,9 @@ def upgrade(migrate_engine):
     c = sqlalchemy.Column('recorded_at', models.PreciseTimestamp(),
                           default=timeutils.utcnow)
     meter.create_column(c)
+
+
+def downgrade(migrate_engine):
+    meta = sqlalchemy.MetaData(bind=migrate_engine)
+    meter = sqlalchemy.Table('meter', meta, autoload=True)
+    meter.drop_column('recorded_at')
