@@ -1,3 +1,7 @@
+# -*- encoding: utf-8 -*-
+#
+# Author: Nejc Saje <nejc.saje@xlab.si>
+#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -21,3 +25,10 @@ def upgrade(migrate_engine):
     alarm = Table('alarm', meta, autoload=True)
     time_constraints = Column('time_constraints', Text())
     alarm.create_column(time_constraints)
+
+
+def downgrade(migrate_engine):
+    meta = MetaData(bind=migrate_engine)
+    alarm = Table('alarm', meta, autoload=True)
+    time_constraints = Column('time_constraints', Text())
+    alarm.drop_column(time_constraints)
