@@ -1,5 +1,8 @@
+# -*- encoding: utf-8 -*-
 #
-# Copyright 2013 Intel Corp.
+# Copyright © 2013 Intel Corp.
+#
+# Author: Yunhong Jiang <yunhong.jiang@intel.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -14,10 +17,25 @@
 # under the License.
 
 import abc
-import collections
-
 import six
 
+from stevedore import extension
+
+
+class TransformerExtensionManager(extension.ExtensionManager):
+
+    def __init__(self, namespace):
+        super(TransformerExtensionManager, self).__init__(
+            namespace=namespace,
+            invoke_on_load=False,
+            invoke_args=(),
+            invoke_kwds={}
+        )
+        self.by_name = dict((e.name, e) for e in self.extensions)
+
+    def get_ext(self, name):
+        return self.by_name[name]
+
 
 @six.add_metaclass(abc.ABCMeta)
 class TransformerBase(object):
@@ -36,42 +54,16 @@ class TransformerBase(object):
         super(TransformerBase, self).__init__()
 
     @abc.abstractmethod
-    def handle_sample(self, sample):
+    def handle_sample(self, context, sample):
         """Transform a sample.
 
+        :param context: Passed from the data collector.
         :param sample: A sample.
         """
 
-    @abc.abstractproperty
-    def grouping_keys(self):
-        """Keys used to group transformer."""
+    def flush(self, context):
+        """Flush samples cached previously.
 
-    @staticmethod
-    def flush():
-        """Flush samples cached previously."""
+        :param context: Passed from the data collector.
+        """
         return []
-
-
-class Namespace(object):
-    """Encapsulates the namespace.
-
-    Encapsulation is done by wrapping the evaluation of the configured rule.
-    This allows nested dicts to be accessed in the attribute style,
-    and missing attributes to yield false when used in a boolean expression.
-    """
-    def __init__(self, seed):
-        self.__dict__ = collections.defaultdict(lambda: Namespace({}))
-        self.__dict__.update(seed)
-        for k, v in six.iteritems(self.__dict__):
-            if isinstance(v, dict):
-                self.__dict__[k] = Namespace(v)
-
-    def __getattr__(self, attr):
-        return self.__dict__[attr]
-
-    def __getitem__(self, key):
-        return self.__dict__[key]
-
-    def __nonzero__(self):
-        return len(self.__dict__) > 0
-    __bool__ = __nonzero__
